#!/usr/bin/perl -w

use strict;
use vars qw($VERSION %IRSSI);

use Irssi;
use Irssi::Irc;

##|
##| Note:
##|
##|     This script is meant to be run on irssi installed on Mac OS X.
##|
##| Help Notes:
##|
##|     This Script is releases as-is.  If you are having problems 
##|     running this script, here is some help I can offer, though I
##|     can't promise this will work so dont complain to me if it doesn't:
##|
##|     * Make sure you have the following set when you compile irssi:
##|         --with-perl=yes
##|         If you already have irssi compiles the easiest way to check
##|         to see if you have perl support is to type:  /script
##|         You will get an 'unknown command' error if you dont, and
##|         possibly something like 'no scripts running' if you do.
##|
##|     * I used DarwinPorts to install irssi.  After DarwinPorts is 
##|         installed, find the Portfile for irssi. It is probably at:
##|             ~/darwinports/dports/irc/irssi/Portfile
##|         Change the text to read:  --with-perl=yes
##|         Then, go to Terminal.app and type:  
##|             sudo port install irssi
##|         If you already had irssi installed, but it is without perl 
##|         support, you can uninstall irssi, and then install it again
##|         after changing the portfile.  To uninstall irssi through
##|         darwinports try:  sudo port uninstall irssi
##|
##|     * irssi scripts need to be placed into the following dir(s):
##|         If you want it to autoload put the file into:  
##|             ~/.irssi/scripts/autorun/
##|         If you want it to be able to load it yourself put it in:
##|             ~/.irssi/scripts/
##|             You will then need to run the command: 
##|                 /script load url.pl
##|         To get to: ~/.irssi hit: cmd-shift-g and type in: ~/.irssi
##|         If you dont already have a scripts dir or autorun dir, you
##|         will need to create them.
##|
##|     * Once installed try the following command:  /url help
##|     
##| Contact Notes:
##| 
##|     * If you want to contact me, feel free to visit #macosx on
##|         irc.freenode.net (/msg NO_PULSE) or christopher@ungeheier.com
##|         or ungeheiercom using your favorite AIM software.  If I
##|         dont respond, dont take it personally, I may not be at my 
##|         computer.  Also, make sure in the first few messages to me
##|         that you got my name from this script so I know not to just
##|         auto-ignore you.  ;)
##|     * If you would like to see more features added to this script
##|         feel free to contact me with the subject 'irssi url Script Features'
##|     * All releases of this script will end up at:
##|         http://irssi.ungeheier.com
##|     * I will continue to update this script when I have the time.  If at
##|         any point I decide to stop development I will make a note in this
##|         file, as well as the url above.
##|     * Until then, I would ask that if you want to edit this file, please 
##|         do not.  If there are errors in this script, I would prefer that 
##|         you inform me, so I can make the needed changed, and upload a new
##|         version to the url above.  I will give you credit for changes, as
##|         well as anyone who requests new features that I havent already planned.
##|
##| Thanks:
##|
##|     I'd like to thank the following people with help on this script:
##|			KrispyKringle - for some insite, and motivation to do this script
##|         #macosx - for testing out this script for me. 
##|
##| Release Notes:
##|
##| 0.0.1: Overhauled Simple Script
##| 0.0.2: Now Grabs /quit /me and regular messages
##| 0.0.3: Cleaned some stuff up, and uploaded to 
##| 			http://irssi.ungeheier.com for the first time.
##| 0.0.4: Added some Anti-Spam Stuff
##| 0.0.5: Added Dupe setting & Cleaned Some Code
##| 0.0.6: Added Clear setting: Will Clear out old URLs everytime
##|			script is loaded.
##| 0.0.7: Cleared out Perl setting and replaced them with irssi triggers
##|			Also fixed a few bugs with trying to print data if user wasn't in a channel
##| 0.0.8: Added 'copy'	command to script which will copy a URL to the clipboard
##|			/url copy #        (example: /url copy 3)
##| 0.0.9: Added Auto setting: Will auto load any URL's said in channel
##| 1.0.0: Added Nick Tracking.
##|			/url list nick  (example: /url list NO_PULSE)
##|			Will return all URLs posted by that nick.  Note: If dupe filter is on, 
##|			you may expect a URL said by one person, but couldve been said by someone
##|			else earlier in the day.
##|			Added the ablility to find URLs that someone has said to you
##|			/url list me
##|			Skipped numbering system and releasing this as 1.0.0 as i'm somewhat
##|			happy with this release.
##| 1.0.1: Added 70+ Top Level Domains instead of the 5 or so it had.
##|

$VERSION = '1.0.1';
%IRSSI = (
    authors     => 'Christopher R. Ungeheier',
    contact     => 'christopher@ungeheier.com',
    name        => 'url',
    description => 'URL Tracker',
    license     => 'If you modify this script please give me credit.',
);

##|
##| Irssi Editable Settings
Irssi::settings_add_str('misc', $IRSSI{'name'} . '_spam_filter', "Yes");
Irssi::settings_add_str('misc', $IRSSI{'name'} . '_browser', "Safari.app");
Irssi::settings_add_str('misc', $IRSSI{'name'} . '_dupe_filter', "Yes");
Irssi::settings_add_str('misc', $IRSSI{'name'} . '_file', "url.txt");
Irssi::settings_add_str('misc', $IRSSI{'name'} . '_clear_trigger', "Yes");
Irssi::settings_add_str('misc', $IRSSI{'name'} . '_auto', "No");

##|
##| Globals
my ($data, $server, $witem) = @_;
my ($url, $text, $nick, $target, $alert) = "";
my @spam_urls = ("macfixer.net","whatsmyip.org","pick-a-site.com");

Irssi::theme_register([
    'url_header', 'url: %_%k%5$0',
    'url_format', 'url: %_$0 %R$1'
]);

if (Irssi::settings_get_str($IRSSI{'name'} . '_clear_trigger') eq "Yes")
{
	clear_urls();
}

Irssi::command_bind url => sub {
	# data - contains the parameters for /url
	# server - the active server in window
	# witem - the active window item (eg. channel, query)
	#         or undef if the window is empty
	($data, $server, $witem) = @_;

	$data =~ s/\s+/ /g;
	my ($todo,$where,$rest) = split(/\s/,$data);

	if($todo =~ /^clear$/i)
	{
		##| Clear out File
		clear_urls();
	} elsif($todo =~ /^last$/i) {
		##| Load up last URL
		$url = last_url();
		browser($url);
	} elsif($todo =~ /^list$/i) {
		##| List URLs
		$nick = $where;
		list_url();
	} elsif($todo =~ /^copy$/i) {
			##| Repost URL
			$url = $where;
			$url = get_url();
			clipboard();
	} elsif($todo =~ /^repost$/i) {
		##| Repost URL
		$url = $where;
		$url = get_url();
		if ($url eq "")
		{
			if ($witem)
			{
				$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_header', "Error: Could Not Find Requested URL #!");
			}
		} else {
			if ($witem)
			{
				my $what = "is looking at: $url";
				$witem->command("/ME $what");
			}
		}		
	} elsif($todo =~ /[0-999]/i) {
		##| Get Specific URL
		$url = $todo;
		$url = get_url();
		if ($url eq "")
		{
			if ($witem)
			{
				$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_header', "Error: Could Not Find Requested URL #!");
			}
		} else {
			browser($url);
		}

	} elsif($todo =~ /^help/i) {
		&help;	
	} else {
		if ($witem)
		{
			$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "Plugin didn't understand your command.");
			$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "type '/url help' for more info");
		}
	}

};

sub clipboard
{
	if ($url eq "")
	{
		if ($witem)
		{
			$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_header', "Error: Could Not Find Requested URL #!");
		}
	} else {
		if ($witem)
		{
			my $clipboard = qq(
				set the clipboard to ("$url" as text)
			);
			my $clipboard = qx(osascript -e '$clipboard');
			$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_header', "Copied '$url' to clipboard.");
		}
	}		

}

sub clear_urls
{
	my $url_file = Irssi::settings_get_str($IRSSI{'name'} . '_file');
	open(URL_FILE, ">$url_file") || print "Could Not Open '$url_file' To Clear.\n";
	close(URL_FILE);
}
sub help 
{
	if ($witem)
	{
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_header', "url $VERSION plugin commands");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_header', "--------Commands--------");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /url clear             : Clears List Of URLs ");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /url list              : Lists URLs ");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /url last              : Loads Last URL into Browser");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /url #                 : Loads specific URL");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /url 1");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /url repost #          : Loads specific URL");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /url repost 1");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /url copy #            : Makes a Copy of specific URL in cipboard");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /url copy 1");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /url help              : shows this help text");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_header', "--------Settings--------");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /set url_browser       : Change The Browser Script Uses To Open URLs");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /set url_browser Firefox.app");
		my $browser = Irssi::settings_get_str($IRSSI{'name'} . '_browser');
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : current setting: '$browser'");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /set url_spam_filter   : Change The Spam Filter Settings (either Yes or No)");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /set url_spam_filter Yes");
		my $spam_filter = Irssi::settings_get_str($IRSSI{'name'} . '_spam_filter');
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : current setting: '$spam_filter'");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /set url_dupe_filter   : Change The Spam Dupe Settings (either Yes or No)");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /set url_dupe_filter Yes");
		my $dupe_filter = Irssi::settings_get_str($IRSSI{'name'} . '_dupe_filter');
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : current setting: '$dupe_filter'");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /set url_file          : Change The File URLs Save To");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /set url_file url.txt");
		my $file = Irssi::settings_get_str($IRSSI{'name'} . '_file');
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : current setting: '$file'");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /set url_clear_trigger : Change Startup Clear Trigger (either Yes or No)");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /set url_clear_trigger Yes");
		my $clear = Irssi::settings_get_str($IRSSI{'name'} . '_clear_trigger');
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : current setting: '$clear'");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', " /set url_auto          : Change Auto Load of URLs (either Yes or No)");
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : example: /set url_auto Yes");
		my $auto = Irssi::settings_get_str($IRSSI{'name'} . '_auto');
		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "                        : current setting: '$auto'");

	}
	
}

sub browser
{
	if ($url =~ /^www/)
	{
		$url = "http://" .$url;
	}
	my $browser = Irssi::settings_get_str($IRSSI{'name'} . '_browser');
	my $browse = qq(
	tell application "$browser"
		 open location "$url"
	end tell
	);
	my $temp = qx(osascript -e '$browse');
}

sub last_url
{
	##| Open File
	my $url_file = Irssi::settings_get_str($IRSSI{'name'} . '_file');
	open(URL_FILE, "$url_file") || print "Could Not Open '$url_file' To Read.\n";
	while (<URL_FILE>)
	{
		chop;
		if ($_ ne "")
		{
			($nick, $url, $alert) = split(/\t/, $_);
		}
	}
	close(URL_FILE);
	return $url;
}

sub list_url
{
	
	if ($witem)
	{
		my ($counter, $my_nick) = "";

		$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_header', "URL $VERSION List use: /url #");
		my $url_file = Irssi::settings_get_str($IRSSI{'name'} . '_file');

		open(URL_FILE, "$url_file") || print "Could Not Open '$url_file' To Read.\n";
		while (<URL_FILE>)
		{
			chop;	chop;
			if ($_ ne "")
			{
				$counter++;
				
				($my_nick, $url, $alert) = split(/\t/, $_);
				if ($nick ne "")
				{
					if ($my_nick eq $nick)
					{
						$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "[$counter]  $my_nick  $url");
					} elsif ($nick eq "me") {
						if ($alert eq "*")
						{
							$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "[$counter]  $my_nick  $url");
						}
					}
				
				} else {
					$witem->printformat(MSGLEVEL_CLIENTCRAP, 'url_format', "[$counter]  $my_nick  $url");
				}
			}
		}
		close(URL_FILE);
	}
}

sub get_url
{
	my ($counter, $url2, $my_nick) = "";
	my $url_file = Irssi::settings_get_str($IRSSI{'name'} . '_file');

	open(URL_FILE, "$url_file") || print "Could Not Open '$url_file' To Read.\n";
	while (<URL_FILE>)
	{
		chop;	chop;
		if ($_ ne "")
		{
			$counter++;
			if ($counter eq $url)
			{
				($my_nick, $url2, $alert) = split(/\t/, $_);
			}
		}
	}
	close(URL_FILE);
	return $url2;
}

sub parse_data
{
	my @urls = ();
	my @dupe_urls = ();
	my ($a, $same, $counter, $url2, $found_spam, $found_dupe, $dupe_url, $spam, $my_nick, $my_url, $alert) = "";
	my $url_file = Irssi::settings_get_str($IRSSI{'name'} . '_file');
	my $url_auto = Irssi::settings_get_str($IRSSI{'name'} . '_auto');
	my $name = Irssi::settings_get_str("nick");

	if ($text =~ /\Q$name/gi)
	{
		$alert = "Yes";
	} else {
		$alert = "";
	}

	$text =~ s/\"+/ /g; #" -- Fixing my editors syntax coloring

	my $tlds = "ac|aero|asn|at|au|be|biz|co|com|coop|ca|cgi|cri|ch|cn|cnf|csiro|edu|de|dk|es|fi|fr|geek|gen|go|gov|govt|gr|hk|hs|id|idv|ie|info|int|is|it|iwi|jobs|jp|kg|kr|li|lu|maori|mil|ms|museum|my|name|ne|net|nl|no|nom|nz|or|org|pe|pt|presse|pro|publ|re|sc|school|se|sg|th|tm|travel|tw|uk";

	if ( ($text =~ /http:\/\//gi) || ($text =~ /www.[a-z0-9]*.($tlds)/gi))
	{
		
		if (Irssi::settings_get_str($IRSSI{'name'} . '_dupe_filter') eq "Yes")
		{
			##| Load file into Array
			open(DUPE_FILE, "$url_file") || print "Could Not Open '$url_file' To Read.\n";

			while (<DUPE_FILE>)
			{
				chop;	chop;
				($my_nick, $my_url) = split(/\t/, $_);
				push(@dupe_urls, $my_url);
			}
			close(DUPE_FILE);
		}
		
		foreach(split(/\s/, $text)) 
		{
			#if ( ($_ =~ /^http\:/) || ($_ =~ /^www.[a-z0-9]*.(com|edu|org|net)/gi))
			if ( ($_ =~ /^http\:/) || ($_ =~ /^www.[a-z0-9]*.($tlds)/gi))
			{
				foreach $a (@urls) 
				{
					if ($_ eq $a) 
					{
						# incase they use the same url on the line.
						$same = 1;
						next;
					}
				}

				if ($same == 0) 
				{
					$same = 0;
					push(@urls, $_);
				}
			}
		}

		open(URL_FILE, ">>$url_file") || print "Could Not Open '$url_file' To Write.\n";

		foreach $a (@urls)
		{

			if (Irssi::settings_get_str($IRSSI{'name'} . '_dupe_filter') eq "Yes")
			{
				foreach $dupe_url (@dupe_urls)
				{
					if ($dupe_url eq $a)
					{
						$found_dupe = "Yes";
						last;
					}
				}	
			}			
		
			if (Irssi::settings_get_str($IRSSI{'name'} . '_spam_filter') eq "Yes")
			{
				foreach $spam (@spam_urls)
				{
					if ($a =~ /$spam/gi)
					{
						$found_spam = "Yes";
						last;
					}
				}	
			}

			if ( ($found_spam ne "Yes") && ($found_dupe ne "Yes") )
			{																						
				if ($alert eq "Yes")
				{
					$alert = "*";
				} 
				print URL_FILE "$nick\t$a\t$alert\r\n";
				
				if (Irssi::settings_get_str($IRSSI{'name'} . '_auto') eq "Yes")
				{
					$url = $a;
					browser($url);
				}
			} else {
				$found_spam = "";
				$dupe_url = "";
			}

		}

		close(URL_FILE);

	}

}

sub event_privmsg 
{
	my ($server, $data, $my_nick, $address) = @_;
	$nick = $my_nick;
	($target, $text) = split(/ :/, $data, 2);
	parse_data($text,$nick);
	
}

Irssi::signal_add("event privmsg", "event_privmsg");

sub event_quit 
{
	my ($server, $data, $my_nick, $address) = @_;
	$nick = $my_nick;
	($target, $text) = split(/:/, $data, 2);
	parse_data($text,$nick);

}

Irssi::signal_add("event quit", "event_quit");

sub event_action 
{
	my ($server, $data, $my_nick, $address) = @_;
	$nick = $my_nick;
	$text = $data;
	parse_data($text,$nick);

}

Irssi::signal_add("message irc action", "event_action");